package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.List;
import java.util.Objects;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.translation.Translatable;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

public interface TranslatableComponent extends BuildableComponent<TranslatableComponent, Builder>, ScopedComponent<TranslatableComponent> {
   @NotNull String key();

   @Contract(
      pure = true
   )
   default @NotNull TranslatableComponent key(final @NotNull Translatable translatable) {
      return this.key(((Translatable)Objects.requireNonNull(translatable, "translatable")).translationKey());
   }

   @Contract(
      pure = true
   )
   @NotNull TranslatableComponent key(final @NotNull String key);

   /** @deprecated */
   @Deprecated
   @NotNull List<Component> args();

   /** @deprecated */
   @Deprecated
   @Contract(
      pure = true
   )
   default @NotNull TranslatableComponent args(final @NotNull ComponentLike... args) {
      return this.arguments(args);
   }

   /** @deprecated */
   @Deprecated
   @Contract(
      pure = true
   )
   default @NotNull TranslatableComponent args(final @NotNull List<? extends ComponentLike> args) {
      return this.arguments(args);
   }

   @NotNull List<TranslationArgument> arguments();

   @Contract(
      pure = true
   )
   @NotNull TranslatableComponent arguments(final @NotNull ComponentLike... args);

   @Contract(
      pure = true
   )
   @NotNull TranslatableComponent arguments(final @NotNull List<? extends ComponentLike> args);

   @Nullable String fallback();

   @Contract(
      pure = true
   )
   @NotNull TranslatableComponent fallback(final @Nullable String fallback);

   default @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.concat(Stream.of(ExaminableProperty.of("key", this.key()), ExaminableProperty.of("arguments", this.arguments()), ExaminableProperty.of("fallback", this.fallback())), BuildableComponent.super.examinableProperties());
   }

   public interface Builder extends ComponentBuilder<TranslatableComponent, Builder> {
      @Contract(
         pure = true
      )
      @NotNull
      default @NotNull Builder key(final @NotNull Translatable translatable) {
         return this.key(((Translatable)Objects.requireNonNull(translatable, "translatable")).translationKey());
      }

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder key(final @NotNull String key);

      /** @deprecated */
      @Deprecated
      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder args(final @NotNull ComponentBuilder<?, ?> arg) {
         return this.arguments(arg);
      }

      /** @deprecated */
      @Deprecated
      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder args(final @NotNull ComponentBuilder<?, ?>... args) {
         return this.arguments((ComponentLike[])args);
      }

      /** @deprecated */
      @Deprecated
      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder args(final @NotNull Component arg) {
         return this.arguments(arg);
      }

      /** @deprecated */
      @Deprecated
      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder args(final @NotNull ComponentLike... args) {
         return this.arguments(args);
      }

      /** @deprecated */
      @Deprecated
      @Contract("_ -> this")
      @NotNull
      default @NotNull Builder args(final @NotNull List<? extends ComponentLike> args) {
         return this.arguments(args);
      }

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder arguments(final @NotNull ComponentLike... args);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder arguments(final @NotNull List<? extends ComponentLike> args);

      @Contract("_ -> this")
      @NotNull
      @NotNull Builder fallback(final @Nullable String fallback);
   }
}
